package pp.monopoly.client.gui;

import com.jme3.math.FastMath;
import com.jme3.math.Vector3f;
import com.jme3.renderer.Camera;

/**
 * Steuert die Kamerabewegung in der Szene.
 */
public class CameraController {
    private final Camera camera;
    private final Vector3f center; // Fokuspunkt der Kamera
    private final float radius;    // Radius der Kreisbewegung
    private final float height;    // Höhe der Kamera über dem Spielfeld
    private final float speed;     // Geschwindigkeit der Kamerabewegung
    private float angle;           // Aktueller Winkel in der Kreisbewegung

    /**
     * Konstruktor für den CameraController.
     *
     * @param camera Die Kamera, die gesteuert werden soll
     * @param center Der Mittelpunkt der Kreisbewegung (Fokuspunkt)
     * @param radius Der Radius der Kreisbewegung
     * @param height Die Höhe der Kamera über dem Fokuspunkt
     * @param speed  Die Geschwindigkeit der Kamerabewegung
     */
    public CameraController(Camera camera, Vector3f center, float radius, float height, float speed) {
        this.camera = camera;
        this.center = center;
        this.radius = radius;
        this.height = height;
        this.speed = speed;
        this.angle = 0; // Starte bei Winkel 0
    }

    /**
     * Aktualisiert die Kameraposition und -ausrichtung.
     *
     * @param tpf Zeit pro Frame
     */
    public void update(float tpf) {
        // Aktualisiere den Winkel basierend auf der Geschwindigkeit
        angle += speed * tpf;
        if (angle >= FastMath.TWO_PI) {
            angle -= FastMath.TWO_PI; // Winkel zurücksetzen, um Überläufe zu vermeiden
        }

        // Berechne die neue Position der Kamera
        float x = center.x + radius * FastMath.cos(angle);
        float z = center.z + radius * FastMath.sin(angle);
        float y = center.y + height;

        // Setze die Kameraposition
        camera.setLocation(new Vector3f(x, y, z));

        // Lasse die Kamera auf den Fokuspunkt blicken
        camera.lookAt(center, Vector3f.UNIT_Y);
    }
}
