package pp.monopoly.client.gui;

import com.jme3.app.Application;
import com.jme3.app.state.BaseAppState;
import com.jme3.material.Material;
import com.jme3.math.ColorRGBA;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Quad;
import com.jme3.texture.Texture;
import com.simsilica.lemur.Axis;
import com.simsilica.lemur.Button;
import com.simsilica.lemur.Container;
import com.simsilica.lemur.Insets3f;
import com.simsilica.lemur.Label;
import com.simsilica.lemur.Selector;
import com.simsilica.lemur.TextField;
import com.simsilica.lemur.component.QuadBackgroundComponent;
import com.simsilica.lemur.component.SpringGridLayout;

import com.simsilica.lemur.core.VersionedList;
import com.simsilica.lemur.core.VersionedReference;
import com.simsilica.lemur.style.ElementId;
import pp.monopoly.client.MonopolyApp;

import java.util.Set;

public class LobbyMenu {

    private final MonopolyApp app;
    private final Container menuContainer;
    private Geometry background;
    private final Selector<String> dropdown;

    public LobbyMenu(MonopolyApp app) {
        this.app = app;

        // Entfernt das CreateGameMenu (inklusive Hintergrund)
        app.getGuiNode().detachAllChildren();

        // Hintergrundbild laden und hinzufügen
        addBackgroundImage();

        QuadBackgroundComponent translucentWhiteBackground =
                new QuadBackgroundComponent(new ColorRGBA(1.0f, 1.0f, 1.0f, 0.5f));

        menuContainer = new Container(new SpringGridLayout(Axis.Y, Axis.X));
        menuContainer.setPreferredSize(new Vector3f(1000, 600, 0)); // Fixed size of the container
        menuContainer.setBackground(translucentWhiteBackground);

        // Create a smaller horizontal container for the label, input field, and spacers
        Container horizontalContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.X, Axis.Y)));
        horizontalContainer.setPreferredSize(new Vector3f(600, 50, 0)); // Adjust container size
        horizontalContainer.setBackground(null);

        Label title = horizontalContainer.addChild(new Label("Startkapital:", new ElementId("label-Bold")));
        title.setFontSize(48);

        // Add a spacer between the title and the input field
        Label spacerBeforeInput = horizontalContainer.addChild(new Label("")); // Invisible spacer
        spacerBeforeInput.setPreferredSize(new Vector3f(20, 1, 0)); // Width of the spacer

        // Add an input field (TextField)
        TextField startingCapital = horizontalContainer.addChild(new TextField(""));
        startingCapital.setPreferredWidth(100); // Set the width of the input field
        startingCapital.setPreferredSize(new Vector3f(150, 50, 0));
        startingCapital.setInsets(new Insets3f(5, 10, 5, 10)); // Add padding around the text inside the field

        // Add a spacer after the input field
        Label spacerAfterInput = horizontalContainer.addChild(new Label("")); // Invisible spacer
        spacerAfterInput.setPreferredSize(new Vector3f(20, 1, 0)); // Width of the spacer

        menuContainer.setLocalTranslation(
                (app.getCamera().getWidth() - menuContainer.getPreferredSize().x) / 2,
                (app.getCamera().getHeight() + menuContainer.getPreferredSize().y) / 2,
                1
        );
        app.getGuiNode().attachChild(menuContainer);

        // Spielerstatus anzeigen
        Container playerListContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.Y, Axis.X)));
        playerListContainer.addChild(new Label("Spieler in der Lobby:"));
        playerListContainer.setBackground(null);
        Label playersLabel = playerListContainer.addChild(new Label("Noch keine Spieler verbunden.")); // Beispieltext

        VersionedList<String> items = new VersionedList<>();
        items.add("Alpha");
        items.add("Beta");
        items.add("Gamma");
        items.add("Back");

        dropdown = new Selector<>(items);
        dropdown.setBackground(new QuadBackgroundComponent(ColorRGBA.Black));
        menuContainer.addChild(dropdown);

        // Add the custom action listener
        addSelectionActionListener(dropdown, this::onDropdownSelectionChanged);



        // Buttons
        Container buttonContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.X, Axis.Y)));
        buttonContainer.setPreferredSize(new Vector3f(400, 50, 0));

        // "Bereit"-Button
        Button readyButton = buttonContainer.addChild(new Button("Bereit"));
        readyButton.setPreferredSize(new Vector3f(120, 40, 0));
        readyButton.addClickCommands(source -> toggleReady(playersLabel));

        // "Zurück"-Button
        Button backButton = buttonContainer.addChild(new Button("Zurück"));
        backButton.setPreferredSize(new Vector3f(120, 40, 0));
        backButton.addClickCommands(source -> goBackToCreateGame());

        // Zentrierung des Containers
        menuContainer.setLocalTranslation(
                (app.getCamera().getWidth() - menuContainer.getPreferredSize().x) / 2,
                (app.getCamera().getHeight() + menuContainer.getPreferredSize().y) / 2,
                1  // Höhere Z-Ebene für den Vordergrund
        );

        app.getGuiNode().attachChild(menuContainer);
    }

    /**
     * Lädt das Hintergrundbild und fügt es als geometrische Ebene hinzu.
     */
    private void addBackgroundImage() {
        Texture backgroundImage = app.getAssetManager().loadTexture("Pictures/unibw-Bib2.png");
        Quad quad = new Quad(app.getCamera().getWidth(), app.getCamera().getHeight());
        background = new Geometry("Background", quad);
        Material backgroundMaterial = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        backgroundMaterial.setTexture("ColorMap", backgroundImage);
        background.setMaterial(backgroundMaterial);
        background.setLocalTranslation(0, 0, -1); // Hintergrundebene

        app.getGuiNode().attachChild(background);
    }

    /**
     * Schaltet den "Bereit"-Status um.
     */
    private void toggleReady(Label playersLabel) {
        // Beispiel-Logik für das Umschalten des Status
        playersLabel.setText("Spielerstatus aktualisiert."); // Beispieltext
    }

    /**
     * Geht zurück zum CreateGameMenu.
     */
    private void goBackToCreateGame() {
        app.getGuiNode().detachChild(menuContainer);
        app.getGuiNode().detachChild(background); // Entfernt das Hintergrundbild
        new CreateGameMenu(app);
    }

    /**
     * Adds a custom action listener to the Selector.
     */
    private void addSelectionActionListener(Selector<String> selector, SelectionActionListener<String> listener) {
        VersionedReference<Set<Integer>> selectionRef = selector.getSelectionModel().createReference();

        app.getStateManager().attach(new BaseAppState() {
            @Override
            public void update(float tpf) {
                if (selectionRef.update()) {
                    String selected = selectionRef.get().toString();
                    System.out.println(selected);
                    listener.onSelectionChanged(selected);
                }
            }

            @Override
            protected void initialize(Application app) {
            }

            @Override
            protected void cleanup(Application app) {
            }

            @Override
            protected void onEnable() {

            }

            @Override
            protected void onDisable() {

            }
        });
    }

    /**
     * Callback for when the dropdown selection changes.
     */
    private void onDropdownSelectionChanged(String selected) {
        System.out.println("Selected: " + selected);
        switch (selected) {
            case "[0]":
                System.out.println("Alpha selected");
                break;
            case "[1]":
                System.out.println("Beta selected");
                break;
            case "[2]":
                System.out.println("Gamma selected");
                break;
            case "[3]":
                goBackToCreateGame();
                break;
            default:
                System.out.println("Unknown selection");
        }
    }

    /**
     * Functional interface for a selection action listener.
     */
    @FunctionalInterface
    private interface SelectionActionListener<T> {
        void onSelectionChanged(T selection);
    }
}
