package pp.monopoly.client.gui;

import com.jme3.material.Material;
import com.jme3.material.RenderState.BlendMode;
import com.jme3.math.ColorRGBA;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Quad;
import com.simsilica.lemur.Button;
import com.simsilica.lemur.Checkbox;
import com.simsilica.lemur.Container;
import com.simsilica.lemur.Label;
import com.simsilica.lemur.Slider;
import com.simsilica.lemur.component.QuadBackgroundComponent;
import com.simsilica.lemur.style.ElementId;
import com.simsilica.lemur.ValueRenderer;

import com.simsilica.lemur.Selector;
import pp.dialog.Dialog;
import pp.monopoly.client.MonopolyApp;

/**
 * SettingsMenu ist ein Overlay-Menü, das durch ESC aufgerufen werden kann.
 */
public class SettingsMenu extends Dialog {
    private final MonopolyApp app;
    private final Geometry overlayBackground;
    private final Container settingsContainer;
    private final Container backgroundContainer;

    public SettingsMenu(MonopolyApp app) {
        super(app.getDialogManager());
        this.app = app;

        // Halbtransparentes Overlay hinzufügen
        overlayBackground = createOverlayBackground();
        app.getGuiNode().attachChild(overlayBackground);

        // Create the background container
        backgroundContainer = new Container();
        backgroundContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.8657f, 0.8735f, 0.8892f, 1.0f))); // Darker background
        app.getGuiNode().attachChild(backgroundContainer);

        // Hauptcontainer für das Menü
        settingsContainer = new Container();
        settingsContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.1f, 0.1f, 0.1f, 0.9f)));



        // Titel
        Label settingsTitle = settingsContainer.addChild(new Label("Einstellungen", new ElementId("settings-title")));
        settingsTitle.setFontSize(48);

        // Effekt-Sound: Slider und Checkbox
        Container effectSoundContainer = settingsContainer.addChild(new Container());
        effectSoundContainer.addChild(new Label("Effekt Sound", new ElementId("label")));
        effectSoundContainer.addChild(new Slider());
        effectSoundContainer.addChild(new Checkbox("Soundeffekte an")).setChecked(true);
        effectSoundContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.4657f, 0.4735f, 0.4892f, 1.0f)));
        // Hintergrundmusik: Slider und Checkbox
        Container backgroundMusicContainer = settingsContainer.addChild(new Container());
        backgroundMusicContainer.addChild(new Label("Hintergrund Musik", new ElementId("label")));
        backgroundMusicContainer.addChild(new Slider());
        backgroundMusicContainer.addChild(new Checkbox("Musik an")).setChecked(true);
        backgroundMusicContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.4657f, 0.4735f, 0.4892f, 1.0f)));

        // Beenden-Button
        Button quitButton = settingsContainer.addChild(new Button("Beenden", new ElementId("menu-button")));
        quitButton.setFontSize(32);
        quitButton.addClickCommands(source -> app.stop());

        float padding = 10; // Padding around the settingsContainer for the background
        backgroundContainer.setPreferredSize(settingsContainer.getPreferredSize().addLocal(padding, padding, 0));


        // Zentriere das Menü
        settingsContainer.setLocalTranslation(
            (app.getCamera().getWidth() - settingsContainer.getPreferredSize().x) / 2,
            (app.getCamera().getHeight() + settingsContainer.getPreferredSize().y) / 2,
            4
        );

        backgroundContainer.setLocalTranslation(
                (app.getCamera().getWidth() - settingsContainer.getPreferredSize().x - padding) / 2,
                (app.getCamera().getHeight() + settingsContainer.getPreferredSize().y+ padding) / 2,
                3
        );

        app.getGuiNode().attachChild(settingsContainer);
    }

    /**
     * Erstellt einen halbtransparenten Hintergrund für das Menü.
     *
     * @return Geometrie des Overlays
     */
    private Geometry createOverlayBackground() {
        Quad quad = new Quad(app.getCamera().getWidth(), app.getCamera().getHeight());
        Geometry overlay = new Geometry("Overlay", quad);
        Material material = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        material.setColor("Color", new ColorRGBA(0, 0, 0, 0.5f)); // Halbtransparent
        material.getAdditionalRenderState().setBlendMode(BlendMode.Alpha);
        overlay.setMaterial(material);
        overlay.setLocalTranslation(0, 0, 0);
        return overlay;
    }

    /**
     * Schließt das Menü und entfernt die GUI-Elemente.
     */
    @Override
    public void close() {
        System.out.println("Schließe SettingsMenu..."); // Debugging-Ausgabe
        app.getGuiNode().detachChild(settingsContainer);  // Entferne das Menü
        app.getGuiNode().detachChild(backgroundContainer); //Entfernt Rand
        app.getGuiNode().detachChild(overlayBackground);  // Entferne das Overlay
        app.setSettingsMenuOpen(false);                  // Menü als geschlossen markieren
        app.unblockInputs();                             // Eingaben wieder aktivieren
        System.out.println("SettingsMenu geschlossen."); // Debugging-Ausgabe
    }


}
