package pp.monopoly.model.fields;

import java.util.ArrayList;
import java.util.List;

import pp.monopoly.game.server.Player;

public class BuildingProperty extends PropertyField {

    private int houses;
    private boolean hotel = false;
    private final int housePrice;
    private final FieldColor color;
    private final int rentFactor1 = 5;
    private final int rentFactor2 = 15;
    private final int rentFactor3 = 40;
    private final int rentFactor4 = 55;
    private final int rentFactorHotel = 70;

    BuildingProperty(String name, int id, int price, int rent, int housePrice, FieldColor color) {
        super(name, id, price, rent);
        this.housePrice = housePrice;
        this.color = color;
    }

    @Override
    public int calcRent() {
        if (hotel) {
            return (int) Math.round(rent*rentFactorHotel/10)*10;
        }
        switch (houses) {
            case 1:
                return (int) Math.round(rent*rentFactor1/10)*10;
            case 2:
                return (int) Math.round(rent*rentFactor2/10)*10;
            case 3:
                return (int) Math.round(rent*rentFactor3/10)*10;
            case 4:
            return (int) Math.round(rent*rentFactor4/10)*10;

            default:
                return rent;
        }
    }

    public boolean buildHouse() {
        if (houses < 4) {
            houses++;
            return true;
        }
        return false;
    }

    public boolean buildHotel() {
        if (hotel) {
            return false;
        }
        hotel = true;
        return true;
    }
    
    public boolean removeHouse() {
        if (houses == 0) {
            return false;
        }
        houses--;
        return true;
    }

    public boolean removeHotel() {
        if (!hotel) {
            return false;
        }
        hotel = false;
        return true;
    }

    @Override
    public void accept(Player player) {
        player.visit(this);
    }

    public List<Integer> getAllRent() {
        List<Integer> list = new ArrayList<>();
        list.add(rent);
        list.add((int) Math.round(rent*rentFactor1/10)*10);
        list.add((int) Math.round(rent*rentFactor2/10)*10);
        list.add((int) Math.round(rent*rentFactor3/10)*10);
        list.add((int) Math.round(rent*rentFactor4/10)*10);
        list.add((int) Math.round(rent*rentFactorHotel/10)*10);
        return list;
    }

    public FieldColor getColor() {
        return color;
    }

    public int getHousePrice() {
        return housePrice;
    }
}
