package pp.monopoly.notification;

import pp.monopoly.model.Board;
import pp.monopoly.model.Item;

/**
 * Event that is triggered when an item is added to a board.
 */
public class ItemAddedEvent {
    private final Item item;
    private final Board board;

    /**
     * Constructs a new ItemAddedEvent.
     *
     * @param item  the item that was added
     * @param board the board to which the item was added
     */
    public ItemAddedEvent(Item item, Board board) {
        this.item = item;
        this.board = board;
    }

    /**
     * Gets the item that was added.
     *
     * @return the added item
     */
    public Item getItem() {
        return item;
    }

    /**
     * Gets the board to which the item was added.
     *
     * @return the board
     */
    public Board getBoard() {
        return board;
    }
}
